@@ -2,7 +2,6 @@ package App::Ack;
 
 use warnings;
 use strict;
-use File::Basename ();
 
 =head1 NAME
 
@@ -10,29 +9,29 @@ App::Ack - A container for functions for the ack program
 
 =head1 VERSION
 
-Version 1.51_01
+Version 1.63_01
 
 =cut
 
 our $VERSION;
 BEGIN {
-    $VERSION = '1.51_01';
+    $VERSION = '1.63_01';
 }
 
 our %types;
 our %mappings;
-our @suffixes;
 our @ignore_dirs;
 our %ignore_dirs;
+our $path_sep;
 our $is_cygwin;
 
 BEGIN {
-    @ignore_dirs = qw( blib CVS RCS SCCS .svn _darcs );
+    @ignore_dirs = qw( blib CVS RCS SCCS .svn _darcs .git );
     %ignore_dirs = map { ($_,1) } @ignore_dirs;
     %mappings = (
         asm         => [qw( s S )],
         binary      => q{Binary files, as defined by Perl's -B op (default: off)},
-        cc          => [qw( c h )],
+        cc          => [qw( c h xs )],
         cpp         => [qw( cpp m h C H )],
         csharp      => [qw( cs )],
         css         => [qw( css )],
@@ -40,35 +39,40 @@ BEGIN {
         haskell     => [qw( hs lhs )],
         html        => [qw( htm html shtml )],
         lisp        => [qw( lisp )],
-        java        => [qw( java )],
+        java        => [qw( java properties )],
         js          => [qw( js )],
-        mason       => [qw( mas )],
+        jsp         => [qw( jsp jspx jhtm jhtml )],
+        make        => q{Makefiles},
+        mason       => [qw( mas mhtml mpl mtxt )],
         ocaml       => [qw( ml mli )],
         parrot      => [qw( pir pasm pmc ops pod pg tg )],
-        perl        => [qw( pl pm pod tt ttml t )],
-        php         => [qw( php phpt htm html )],
+        perl        => [qw( pl pm pod t )],
+        php         => [qw( php phpt php3 php4 php5 )],
         python      => [qw( py )],
-        ruby        => [qw( rb rhtml rjs )],
+        ruby        => [qw( rb rhtml rjs rxml )],
         scheme      => [qw( scm )],
         shell       => [qw( sh bash csh ksh zsh )],
         sql         => [qw( sql ctl )],
-        tex         => [qw( tex )],
-        tt          => [qw( tt tt2 )],
+        tcl         => [qw( tcl )],
+        tex         => [qw( tex cls sty )],
+        tt          => [qw( tt tt2 ttml )],
+        vb          => [qw( bas cls frm ctl vb resx )],
         vim         => [qw( vim )],
         yaml        => [qw( yaml yml )],
         xml         => [qw( xml dtd xslt )],
     );
 
-    my %suffixes;
+    use File::Spec ();
+    $path_sep = File::Spec->catfile( '', '' );
+    $path_sep = quotemeta( $path_sep );
+
     while ( my ($type,$exts) = each %mappings ) {
         if ( ref $exts ) {
             for my $ext ( @{$exts} ) {
                 push( @{$types{$ext}}, $type );
-                ++$suffixes{"\\.$ext"};
             }
         }
     }
-    @suffixes = keys %suffixes;
 
     $is_cygwin = ($^O eq 'cygwin');
 }
@@ -81,24 +85,6 @@ No user-serviceable parts inside.  F<ack> is all that should use this.
 
 =head1 FUNCTIONS
 
-=head2 is_filetype( $filename, $filetype )
-
-Asks whether I<$filename> is of type I<$filetype>.
-
-=cut
-
-sub is_filetype {
-    my $filename = shift;
-    my $wanted_type = shift;
-
-    for my $maybe_type ( filetypes( $filename ) ) {
-        return 1 if $maybe_type eq $wanted_type;
-    }
-
-    return;
-}
-
-
 =head2 skipdir_filter
 
 Standard filter to pass as a L<File::Next> descend_filter.  It
@@ -127,23 +113,20 @@ even under -a.
 sub filetypes {
     my $filename = shift;
 
-    return '-ignore' if $filename =~ /~$/;
+    return '-ignore' unless is_searchable( $filename );
 
-    # Pass our $filename in lowercase so we match lowercase filenames
-    my ($filebase,$dirs,$suffix) = File::Basename::fileparse( lc $filename, @suffixes );
-
-    return '-ignore' if $filebase =~ /^#.+#$/;
-    return '-ignore' if $filebase =~ /^core\.\d+$/;
+    return 'make' if $filename =~ m{$path_sep?Makefile$}io;
 
     # If there's an extension, look it up
-    if ( $suffix ) {
-        $suffix =~ s/^\.//; # Drop the period that File::Basename needs
-        my $ref = $types{lc $suffix};
+    if ( $filename =~ m{\.([^\.$path_sep]+)$}o ) {
+        my $ref = $types{lc $1};
         return @{$ref} if $ref;
     }
 
-    return unless -e $filename;
+    # At this point, we can't tell from just the name.  Now we have to
+    # open it and look inside.
 
+    return unless -e $filename;
     # From Elliot Shank:
     #     I can't see any reason that -r would fail on these-- the ACLs look
     #     fine, and no program has any of them open, so the busted Windows
@@ -152,7 +135,7 @@ sub filetypes {
     # So, for cygwin, don't bother trying to check for readability.
     if ( !$is_cygwin ) {
         if ( !-r $filename ) {
-            warn _my_program(), ": $filename: Permission denied\n";
+            App::Ack::warn( "$filename: Permission denied" );
             return;
         }
     }
@@ -162,24 +145,40 @@ sub filetypes {
     # If there's no extension, or we don't recognize it, check the shebang line
     my $fh;
     if ( !open( $fh, '<', $filename ) ) {
-        warn _my_program(), ": $filename: $!\n";
+        App::Ack::warn( "$filename: $!" );
         return;
     }
     my $header = <$fh>;
-    close $fh;
-    return unless defined $header;
+    if ( not close $fh ) {
+        App::Ack::warn( "$filename: $!" );
+        return;
+    }
+
     if ( $header =~ /^#!/ ) {
-        return 'perl'   if $header =~ /\bperl/;
-        return 'php'    if $header =~ /\bphp\b/;
-        return 'python' if $header =~ /\bpython\b/;
-        return 'ruby'   if $header =~ /\bruby\b/;
-        return 'shell'  if $header =~ /\b(ba|c|k|z)?sh\b/;
+        return $1       if $header =~ /\b(ruby|p(erl|hp|ython))\b/;
+        return 'shell'  if $header =~ /\b(?:ba|c|k|z)?sh\b/;
     }
     return 'xml' if $header =~ /<\?xml /;
 
     return;
 }
 
+=head2 is_searchable( $filename )
+
+Returns true if the filename is one that we can search, and false
+if it's one that we should ignore like a coredump or a backup file.
+
+=cut
+
+sub is_searchable {
+    my $filename = shift;
+
+    return if $filename =~ /~$/;
+    return if $filename =~ m{$path_sep?(?:#.+#|core\.\d+)$}o;
+
+    return 1;
+}
+
 =head2 options_sanity_check( %opts )
 
 Checks for sane command-line options.  For example, I<-l> doesn't
@@ -191,8 +190,13 @@ sub options_sanity_check {
     my %opts = @_;
     my $ok = 1;
 
+    # List mode doesn't make sense with any of these
     $ok = 0 if _option_conflict( \%opts, 'l', [qw( A B C o group )] );
+
+    # XXX This should work, I would think.
     $ok = 0 if _option_conflict( \%opts, 'l', [qw( m )] );
+
+    # File-searching is definitely irrelevant on these
     $ok = 0 if _option_conflict( \%opts, 'f', [qw( A B C o m group )] );
 
     return $ok;
@@ -206,9 +210,9 @@ sub _option_conflict {
     return if not defined $opts->{$used};
 
     my $bad = 0;
-    for ( @$exclusives ) {
-        if ( defined $opts->{$_} ) {
-            print "The ", _opty($_), " option cannot be used with the ", _opty($used), " option.\n";
+    for my $opt ( @{$exclusives} ) {
+        if ( defined $opts->{$opt} ) {
+            print 'The ', _opty($opt), ' option cannot be used with the ', _opty($used), " option.\n";
             $bad = 1;
         }
     }
@@ -221,7 +225,28 @@ sub _opty {
     return length($opt)>1 ? "--$opt" : "-$opt";
 }
 
+=head2 warn( @_ )
+
+Put out an ack-specific warning.
+
+=cut
+
+sub warn { ## no critic (ProhibitBuiltinHomonyms)
+    return CORE::warn( _my_program(), ': ', @_, "\n" );
+}
+
+=head2 die( @_ )
+
+Die in an ack-specific way.
+
+=cut
+
+sub die { ## no critic (ProhibitBuiltinHomonyms)
+    return CORE::die( _my_program(), ': ', @_, "\n" );
+}
+
 sub _my_program {
+    require File::Basename;
     return File::Basename::basename( $0 );
 }
 
@@ -258,6 +283,7 @@ sub show_help {
 
     print <<"END_OF_HELP";
 Usage: ack [OPTION]... PATTERN [FILES]
+
 Search for PATTERN in each source file in the tree from cwd on down.
 If [FILES] is specified, then only those files/directories are checked.
 ack may also search STDIN, but only if no FILES are specified, or if
@@ -268,57 +294,61 @@ Default switches may be specified in ACK_OPTIONS environment variable.
 Example: ack -i select
 
 Searching:
-    -i              Ignore case distinctions
-    -v              Invert match: select non-matching lines
-    -w              Force PATTERN to match only whole words
-    -Q              Quote all metacharacters; expr is literal
+  -i, --ignore-case     Ignore case distinctions
+  -v, --invert-match    Invert match: select non-matching lines
+  -w, --word-regexp     Force PATTERN to match only whole words
+  -Q, --literal         Quote all metacharacters; expr is literal
 
 Search output:
-    -l              Only print filenames containing matches
-    -o              Show only the part of a line matching PATTERN
-                    (turns off text highlighting)
-    --output=expr   Output the evaluation of expr for each line
-                    (turns off text highlighting)
-    -m=NUM          Stop after NUM matches
-    -H              Print the filename for each match
-    -h              Suppress the prefixing filename on output
-    -c, --count     Show number of lines matching per file
+  -l, --files-with-matches
+                        Only print filenames containing matches
+  -L, --files-without-match
+                        Only print filenames with no match
+  -o                    Show only the part of a line matching PATTERN
+                        (turns off text highlighting)
+  --output=expr         Output the evaluation of expr for each line
+                        (turns off text highlighting)
+  -m, --max-count=NUM   Stop after NUM matches
+  -H, --with-filename   Print the filename for each match
+  -h, --no-filename     Suppress the prefixing filename on output
+  -c, --count           Show number of lines matching per file
 
-    --group         Group matches by file name.
-                    (default: on when used interactively)
-    --nogroup       One result per line, including filename, like grep
-                    (default: on when the output is redirected)
+  --group               Group matches by file name.
+                        (default: on when used interactively)
+  --nogroup             One result per line, including filename, like grep
+                        (default: on when the output is redirected)
 
-    --[no]color     Highlight the matching text (default: on unless
-                    output is redirected, or on Windows)
+  --[no]color           Highlight the matching text (default: on unless
+                        output is redirected, or on Windows)
 
 Context control:
-    -B, --before-context=NUM
-    -A, --after-context=NUM
-    -C, --context=NUM
-                    print NUM lines of context before and/or after
-                    matching lines
+  -B, --before-context=NUM
+  -A, --after-context=NUM
+  -C, --context=NUM
+                        Print NUM lines of context before and/or after
+                        matching lines
 
 File finding:
-    -f              Only print the files found, without searching.
-                    The PATTERN must not be specified.
-    --sort-files    Sort the found files lexically.
+  -f                    Only print the files found, without searching.
+                        The PATTERN must not be specified.
+  --sort-files          Sort the found files lexically.
 
 File inclusion/exclusion:
-    -n              No descending into subdirectories
-    -a, --all       All files, regardless of extension (but still skips
-                    $ignore_dirs dirs)
-    --perl          Include only Perl files.
-    --type=perl     Include only Perl files.
-    --noperl        Exclude Perl files.
-    --type=noperl   Exclude Perl files.
-                    See "ack --help type" for supported filetypes.
+  -n                    No descending into subdirectories
+  -a, --all             All files, regardless of extension (but still skips
+                        $ignore_dirs dirs)
+  --perl                Include only Perl files.
+  --type=perl           Include only Perl files.
+  --noperl              Exclude Perl files.
+  --type=noperl         Exclude Perl files.
+                        See "ack --help type" for supported filetypes.
+  --[no]follow          Follow symlinks.  Default is off.
 
 Miscellaneous:
-    --help          This help
-    --man           Man page
-    --version       Display version & copyright
-    --thpppt        Bill the Cat
+  --help                This help
+  --man                 Man page
+  --version             Display version & copyright
+  --thpppt              Bill the Cat
 END_OF_HELP
 
     return;
@@ -344,7 +374,8 @@ Note that some extensions may appear in multiple types.  For example,
 
 END_OF_HELP
 
-    for my $type ( sort( filetypes_supported() ) ) {
+    my @types = filetypes_supported();
+    for my $type ( sort @types ) {
         next if $type =~ /^-/; # Stuff to not show
         my $ext_list = $mappings{$type};
 
